module M508Emu(GPIO,LED,KEY);
	inout [7:0] GPIO;
	output	[7:0]	LED;
	input	[3:0]	KEY	;
	wire Qin,Qout,Q;
	wire [15:0] cmd;
	wire [8:0] adr;
	wire fosc,reset,cmd_freq,cmd_flash;
	wire [7:0] wdeb;
	wire [4:0] inst;
	wire wf;
	wire wr_pc;
	wire [8:0] pc_in;
	wire wr_tmr;
	wire [7:0] tim_out;
	wire [7:0] opt;
	wire [7:0] db_out;
	wire wr_wreg;
	wire [7:0] wreg;
	wire [7:0] db_in;
	wire cin,cout,dc,zf,wr_fr,wr_cf,wr_dc,wr_zf,cmd_skip,wr_opt,wr_tris;
	wire [7:0] gp_in;
	wire [7:0] gp_out;
	wire [7:0] tris_out;
	
	assign cmd_flash=cmd_freq | cmd_skip | wr_pc;
	
	/* ============================================================ 
	//Slow Clock
	mOsc1s OSC2(reset,fosc,Qin,Qout);
	assign Q=Qout;
	=============================================================== */
	/* ============================================================ 
	//Step Debug
	mStep step(reset,fosc,Qin,KEY[3],Qout);
	assign Q=Qout;
	=============================================================== */
	///* ============================================================
	//Normal
	mOscDiv OSC2(reset,fosc,Qin,Qout);
	assign Q=Qout;
	//===============================================================

	//Power On Reset ==============================================
	mReset Rst(fosc,Qin,reset);
	//assign reset=~KEY[3];	//Hardware  Reset======================

	//Program Memory Module =======================================
	mProgMem pmem(fosc,reset,Qin,adr,cmd,Q,cmd_flash);

	//Command Proceccer ===========================================
	mCmdProc Cproc(cmd[11:0],cmd_freq,inst);

	//Program Counter =============================================
	mPCounter PC(reset,fosc,Q,inst,cmd[11:0],adr,wr_pc,pc_in);

	//Timer 0 =====================================================
	mTimer0 tmr0(reset,fosc,Q,opt[3:0],wr_tmr,db_out,tim_out);

	//W-Reg =======================================================
	mWreg w_reg(reset,fosc,Q,wr_wreg,db_out,wreg);
	
	//ALU Module ==================================================
	mALU alu(inst,cmd[11:0],wreg,db_in,cin,db_out,cout,dc,zf,wr_fr,wr_wreg,wr_cf,wr_dc,wr_zf,cmd_skip,wr_opt,wr_tris);

	//File Register ===============================================
	mRegFile regfile(reset,fosc,Q,cmd[4:0],db_out,db_in,cout,cin,dc,zf,wr_cf,wr_dc,wr_zf,wr_fr,gp_in,gp_out,wr_tris,tris_out,adr,pc_in,wr_pc,tim_out,wr_tmr,opt,wr_opt);	

	//GPIO Setting ================================================
	assign GPIO[0]=(tris_out[0]) ? 1'bz : gp_out[0];
	assign GPIO[1]=(tris_out[1]) ? 1'bz : gp_out[1];
	assign GPIO[2]=(tris_out[2]) ? 1'bz : gp_out[2];
	assign GPIO[3]=(tris_out[3]) ? 1'bz : gp_out[3];
	assign GPIO[4]=(tris_out[4]) ? 1'bz : gp_out[4];
	assign GPIO[5]=(tris_out[5]) ? 1'bz : gp_out[5];
	assign GPIO[6]=(tris_out[6]) ? 1'bz : gp_out[6];
	assign GPIO[7]=(tris_out[7]) ? 1'bz : gp_out[7];

	//for Standerd I/O ==========================================
	assign LED=gp_out;
	
	//Standerd Input ============================================
	assign gp_in={4'h0,KEY};
	//for Bi-Direction ==========================================
	//assign gp_in=GPIO;

	//for LED debug==============================================
	/*
	assign LED=(KEY==4'he) ? ~cmd[7:0] : 
		(KEY==4'hd) ? ~cmd[15:8] :
		(KEY==4'hb) ? ~db_out :
		(KEY==4'ha) ? ~db_in :
		(KEY==4'hc) ? ~wreg :
		(KEY==4'h8) ? ~{cout,dc,zf,inst} :
		(KEY==4'h9) ? ~tim_out :~adr[7:0];
	*/
	//==========================================================
endmodule
