//Register Files

//Status Register

module mStatus(reset,clk,Q,wr_zf,wr_dc,wr_cf,wr_fr,zf,dc,cf,datin,datout);
	input reset,clk;
	input Q;
	input wr_zf,wr_dc,wr_cf,wr_fr,zf,dc,cf;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] streg;
	
	assign datout=streg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			streg<=8'h18;
		end else if(Q) begin
			if(wr_fr) begin
				streg<=datin;
			end else begin
				if(wr_zf)
					streg[2]<=zf;
				if(wr_dc)
					streg[1]<=dc;
				if(wr_cf)
					streg[0]<=cf;
			end
		end
	end
endmodule
//Option Register
module mOption(reset,clk,Q,wr_opt,datin,datout);
	input reset,clk;
	input Q;
	input wr_opt;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] optreg;
	assign datout=optreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			optreg<=8'hff;
		end else if(Q) begin
			if(wr_opt) begin
				optreg<=datin;
			end
		end
	end
endmodule

//TRIS Register
module mTris(reset,clk,Q,wr_tris,datin,datout);
	input reset,clk;
	input Q;
	input wr_tris;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] trisreg;
	assign datout=trisreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			trisreg<=8'hff;
		end else if(Q) begin
			if(wr_tris) begin
				trisreg<=datin;
			end
		end
	end
endmodule


//FSR Register
module mFsr(reset,clk,Q,wr_fsr,datin,datout);
	input reset,clk;
	input Q;
	input wr_fsr;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] fsrreg;
	assign datout=fsrreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			fsrreg<=8'hff;
		end else if(Q) begin
			if(wr_fsr) begin
				fsrreg<=datin;
			end
		end
	end
endmodule

//Gpio Reg
module mGpio(reset,clk,Q,wr_gpio,datin,datout);
	input reset,clk;
	input Q;
	input wr_gpio;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] gpreg;
	assign datout=gpreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			gpreg<=8'h00;
		end else if(Q) begin
			if(wr_gpio) begin
				gpreg<=datin;
			end
		end
	end
endmodule

module mRegFile(
	reset,clk,Q,	//Reset,fosc,Q state
	fadr,			//File Reg Address
	//ALU Interface
	datin,
	datout,
	cin,cout,dc,zf,wr_cf,wr_dc,wr_zf,wr_fr,
	//GPIO
	gp_in,gp_out,wr_tris,tris_out,
	//Program Counter
	pc_in,pc_out,wr_pc,
	//Timer
	timer_in,wr_tmr,
	//OPtion Reg 
	opt_out,wr_opt
	);
	input reset,clk;
	input Q;
	input [4:0] fadr;
	input [7:0] datin;
	output [7:0] datout;
	input cin;
	output cout;
	input dc,zf,wr_cf,wr_dc,wr_zf,wr_fr; 
	input [7:0] gp_in;	//input from GPIO pin
	output [7:0] gp_out;	//Output to GPIO
	input wr_tris;		//Write TRIS 
	output [7:0] tris_out; //TRIS Out
	input [8:0] pc_in;	//Program Counter
	output [8:0] pc_out;
	output wr_pc;
	input [7:0] timer_in;
	output wr_tmr;
	output [7:0] opt_out;
	input wr_opt;
	wire [4:0] iadr;	//internal Address
	wire [7:0] fsr_out;	//FSR Reg
	wire [7:0] st_out;	//Status Reg
	reg [7:0] ram[7:31]; //RAM
	//Write Signals
	wire wr_st,wr_fsr,wr_gpio,wr_ram;
	
	assign cout=st_out[0];	
	assign pc_out={st_out[5],datin};
	assign iadr=(fadr[4:0]==5'b00000) ? fsr_out[4:0] : fadr;
	assign datout=(iadr==5'h1) ? timer_in :
		(iadr==5'h2) ? pc_in[7:0] :
		(iadr==5'h3) ? st_out :
		(iadr==5'h4) ? fsr_out :
		(iadr==5'h6) ? gp_in :
		((iadr==5'h7)||(iadr[3])||(iadr[4])) ? ram[iadr] : 8'h0;
	//Write signals
	assign wr_tmr=(iadr==5'h1) ? wr_fr : 1'b0;
	assign wr_pc=(iadr==5'h2) ? wr_fr : 1'b0;
	assign wr_st=(iadr==5'h3) ? wr_fr : 1'b0;
	assign wr_fsr=(iadr==5'h4) ? wr_fr : 1'b0;
	assign wr_gpio=(iadr==5'h6) ? wr_fr : 1'b0;
	assign wr_ram=((iadr==5'h7)||(iadr[3])||(iadr[4])) ? wr_fr : 1'b0;
	
	always@(posedge clk) begin
		if(Q) begin
			if(wr_ram) begin
				ram[iadr]<=datin;
			end
		end
	end
	mStatus streg(reset,clk,Q,wr_zf,wr_dc,wr_cf,wr_st,zf,dc,cin,datin,st_out);
	mOption optreg(reset,clk,Q,wr_opt,datin,opt_out);
	mTris trosreg(reset,clk,Q,wr_tris,datin,tris_out);
	mFsr fsrreg(reset,clk,Q,wr_fsr,datin,fsr_out);
	mGpio gpioreg(reset,clk,Q,wr_gpio,datin,gp_out);
endmodule
