`timescale 1 ps / 1 ps

// ALU Module

//4bit Adder
module mAdder4(dat1,dat2,cin,out,cout);
	input [3:0] dat1;
	input [3:0] dat2;
	input cin;
	output [3:0] out;
	output cout;
	wire [4:0] wout;
	
	assign wout=dat1+dat2+cin;
	assign cout=wout[4];
	assign out=wout[3:0];
endmodule

//8bit Adder
module mAdder8(dat1,dat2,cin,out,cout,dc);
	input [7:0] dat1;
	input [7:0] dat2;
	input cin;
	output [7:0] out;
	output cout,dc;
	wire wdc;
	
	assign dc=wdc;
	mAdder4 add4a(dat1[3:0],dat2[3:0],cin,out[3:0],wdc);
	mAdder4 add4b(dat1[7:4],dat2[7:4],wdc,out[7:4],cout);
endmodule

//BitMask
module mBitMask(bitin,mask);
	input [2:0] bitin;
	output [7:0] mask;
	reg [7:0] maskdat;
	assign mask=maskdat;
	always @(bitin) begin
		case (bitin)
			3'd0: maskdat=8'h01;
			3'd1: maskdat=8'h02;
			3'd2: maskdat=8'h04;
			3'd3: maskdat=8'h08;
			3'd4: maskdat=8'h10;
			3'd5: maskdat=8'h20;
			3'd6: maskdat=8'h40;
			3'd7: maskdat=8'h80;
		endcase
	end
endmodule

//WReg
module mWreg(reset,clk,Q,write,datin,datout);
	input reset,clk;
	input Q;
	input write;
	input [7:0] datin;
	output [7:0] datout;
	reg [7:0] wreg;
	assign datout=wreg;
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			wreg<=8'h00;
		end else if(Q) begin
			if(write) begin
				wreg<=datin;
			end
		end
	end
endmodule


		
//ALU Module
module mALU(cmd,wreg,datin,cin,datout,cout,dc,zf,wr_f,wr_w,wr_c,wr_dc,wr_zf,cmd_skip,wr_opt,wr_tris);
	input [11:0] cmd;
	input [7:0] wreg;
	input [7:0] datin;
	input cin;
	output [7:0] datout;
	wire [7:0] addout;
	wire [7:0] w_wreg;
	wire [7:0] w_datin;
	wire [7:0] w_datout;
	wire [7:0] bitmask;
	wire acin,acout;
	output cout,dc,zf,wr_f,wr_w,wr_c,wr_dc,wr_zf,cmd_skip,wr_opt,wr_tris;

	wire
	ADDWF, ANDWF, CLRF, CLRW, COMF, DECF, DECFSZ, INCF,  INCFSZ,
	IORWF, MOVF,  MOVWF, RLF,  RRF,  SUBWF,SWAPF, XORWF, BCF,  BSF, 
	BTFSC, BTFSS, ANDLW, CALL, GOTO, IORLW,MOVLW, RETLW, XORLW, 
	OPTION,CLRWDT,TRIS,  NOP ;

	mCmdDecode2 cmd_dec (
		.cmd(cmd),
		.addwf(ADDWF),
		.andwf(ANDWF),
		.clrf(CLRF),
		.clrw(CLRW),
		.comf(COMF),
		.decf(DECF),
		.decfsz(DECFSZ),
		.incf(INCF),
		.incfsz(INCFSZ),
		.iorwf(IORWF),
		.movf(MOVF),
		.movwf(MOVWF),
		.rlf(RLF),
		.rrf(RRF),
		.subwf(SUBWF),
		.swapf(SWAPF),
		.xorwf(XORWF),
		.bcf(BCF),
		.bsf(BSF),
		.btfsc(BTFSC),
		.btfss(BTFSS),
		.andlw(ANDLW),
		.call(CALL),
		.goto(GOTO),
		.iorlw(IORLW),
		.movlw(MOVLW),
		.retlw(RETLW),
		.xorlw(XORLW), 
		.option(OPTION),
		.clrwdt(CLRWDT),
		.tris(TRIS),
		.nop(NOP)
	) ;

	//Adder(ADD/SUB)
	assign acin=(SUBWF) ? 1'b1 : 1'b0;
	assign w_wreg=(SUBWF) ? ~wreg :
		((INCF)||(INCFSZ)) ? 8'h01 :
		((DECF)||(DECFSZ)) ? 8'hff :
		((BSF)||(BTFSC)||(BTFSS)) ? bitmask :
		(BCF) ? ~bitmask :	wreg;
	assign w_datin=(ANDLW | IORLW | MOVLW | RETLW | XORLW) ? cmd[7:0] : datin;
	//Adder
	mAdder8 adder(w_wreg,w_datin,acin,addout,acout,dc);
	//bitmask
	mBitMask bmask(cmd[7:5],bitmask);
	
	assign cout=
		(RLF) ? datin[7] :
		(RRF) ? datin[0] : acout;
	assign datout=w_datout;
	assign zf=(w_datout==8'h00) ? 1'b1 : 1'b0;
	
	//ALU Main
	assign w_datout=
		(ADDWF | SUBWF | INCF | INCFSZ | DECF | DECFSZ) ? addout  :	//Addtion Function
		(ANDWF | ANDLW | BCF | BTFSC | BTFSS) ? (w_wreg & w_datin) :				//AND
		(CLRF | CLRW) ? 8'h00  :		//CLR
		((IORWF | BSF | IORLW))	? (w_datin | w_wreg) :	//OR
		(RLF) ? {w_datin[6:0],cin} :		//SHIFT LEFT
		(RRF) ? {cin,w_datin[7:1]} :		//SHIFT RIGHT
		(XORWF | XORLW) ? (w_wreg ^ w_datin) :	//XOR
		(SWAPF) ? {w_datin[3:0],w_datin[7:4]} : //Swap
		(COMF) ? ~w_datin :	//Complement
		(MOVF | MOVLW | RETLW) ? w_datin : w_wreg;	//File Reg :othere=wreg
	//assign ,
	assign wr_f= (
		( (ADDWF | ANDWF | CLRF | COMF | DECF | DECFSZ | INCF |
		   INCFSZ | IORWF | MOVF | RLF | RRF | SUBWF | SWAPF | XORWF) & cmd[5])|
		(BCF | BSF | MOVWF)
	)? 1'b1 : 1'b0;
	assign wr_w= (
		( (ADDWF | ANDWF | CLRW | COMF | DECF | DECFSZ | INCF | INCFSZ |
		   IORWF | MOVF | RLF | RRF | SUBWF | SWAPF | XORWF) & ~cmd[5])|
		(ANDLW | IORLW | MOVLW | RETLW | XORLW)
	)? 1'b1 : 1'b0;
	assign wr_c=(ADDWF | RLF | RRF | SUBWF)? 1'b1 : 1'b0;
	assign wr_dc=(ADDWF | SUBWF)? 1'b1 : 1'b0;
	assign wr_zf=(ADDWF | ANDWF | CLRF | CLRW | COMF | DECF | INCF | 
	              IORWF | MOVF | SUBWF | XORWF | ANDLW | IORLW | XORLW)? 1'b1 : 1'b0;
	//assign cmd_skip=((GOTO)||(CALL)||(RETLW)||(((DECFSZ)||(INCFSZ)||(BTFSC))&&(zf))||((BTFSS)&&(!zf))) ? 1'b1 :1'b0;
	//GOTO,CALL,RET using CmdProc
	assign cmd_skip= (
		((DECFSZ | INCFSZ | BTFSC)& zf)|
		(BTFSS & ~zf)
	)? 1'b1 :1'b0;
	assign wr_tris=(TRIS)? 1'b1 : 1'b0;
	assign wr_opt=(OPTION)? 1'b1 : 1'b0;
endmodule
