`timescale 1 ps / 1 ps

module mStack(reset,clk,Q,write,read,datin,datout);
	input reset,clk,write,read;
	input Q;
	input [8:0] datin;
	output [8:0] datout;
	reg [8:0] stack1;
	reg [8:0] stack2;
	
	assign datout=stack1;
	
	//Stack1
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			stack1<=9'h0;
		end else begin
			if(Q & write)
				stack1<=datin;
			else if(Q & read)
				stack1<=stack2;
		end
	end
	//Stack2
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			stack2<=9'h0;
		end else begin
			if(Q & write)
				stack2<=stack1;
			else if(Q & read)
				stack2<=9'h0;
		end
	end
endmodule

//Program Counter
module mPCounter(reset,clk,Q,cmd,adr,wr_pc,wr_dat);
	input reset,clk;
	input  Q;			//Start State
	input [11:0] cmd;	//Command Code(data)
	output [8:0] adr;	//Memory Address
	input wr_pc;		//Write OP to PC
	input [8:0] wr_dat;	//Write Data
	wire call,ret;
	reg [8:0] pc_reg;	//Program Counter
	wire [8:0] sdout;	//Stack Data Output

	wire
	GOTO, CALL, RETLW ;

	mCmdDecode2 cmd_dec (
		.cmd(cmd),
		.goto(GOTO),
		.call(CALL),
		.retlw(RETLW)
	) ;

	assign call=(CALL) ? 1'b1 : 1'b0;
	assign ret =(RETLW)? 1'b1 : 1'b0;
	assign adr=pc_reg;
	
	//Program Counter
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			pc_reg<=9'h0;
		end 
		else if(Q) begin
			if(GOTO) begin
				pc_reg<=cmd[8:0];
			end else if(CALL) begin
				pc_reg<={1'b0,cmd[7:0]};
			end else if(RETLW) begin
				pc_reg<=sdout;
			end else if(wr_pc) begin
				pc_reg<=wr_dat;
			end else begin
				pc_reg<=pc_reg+1'b1;
			end
		end
	end
	
	//Stack
	mStack stk(reset,clk,Q,call,ret,pc_reg,sdout);
endmodule


//CommandProcesser
module mCmdProc(cmd,cmd_flash);
	input [11:0] cmd;
	output cmd_flash;

	wire
	GOTO, CALL, RETLW ;

	mCmdDecode2 cmd_dec (
		.cmd(cmd),
		.goto(GOTO),
		.call(CALL),
		.retlw(RETLW)
	) ;

	assign cmd_flash=(GOTO | CALL | RETLW) ? 1'b1 : 1'b0;
endmodule
