`include "Instruction.v"

module mCmdDecord(cmd,inst);
	input [11:0] cmd;
	output [4:0] inst;
	assign inst=(cmd[11:6]==6'b0001_11) ? `ADDWF :
			(cmd[11:6]==6'b0001_01)   ? `ANDWF :
			(cmd[11:5]==7'b0000_011) ? `CLRF :
			(cmd==12'b0000_0100_0000) ? `CLRW :
			(cmd[11:6]==6'b0010_01)  ? `COMF :
			(cmd[11:6]==6'b0000_11)  ? `DECF :
			(cmd[11:6]==6'b0010_11)  ? `DECFSZ:
			(cmd[11:6]==6'b0010_10)  ? `INCF :
			(cmd[11:6]==6'b0011_11)  ? `INCFSZ :
			(cmd[11:6]==6'b0001_00)  ? `IORWF :
			(cmd[11:6]==6'b0010_00)  ? `MOVF :
			(cmd[11:5]==7'b0000_001) ? `MOVWF :
			(cmd[11:6]==6'b0011_01)  ? `RLF :
			(cmd[11:6]==6'b0011_00)  ? `RRF :
			(cmd[11:6]==6'b0000_10)  ? `SUBWF :
			(cmd[11:6]==6'b0011_10)  ? `SWAPF :
			(cmd[11:6]==6'b0001_10)  ? `XORWF :
			(cmd[11:8]==4'b0100)  	? `BCF :
			(cmd[11:8]==4'b0101)	? `BSF :
			(cmd[11:8]==4'b0110)	? `BTFSC :
			(cmd[11:8]==4'b0111)	? `BTFSS :
			(cmd[11:8]==4'b1110)	? `ANDLW :
			(cmd[11:8]==4'b1001)	? `CALL :
			(cmd[11:9]==3'b101)		? `GOTO :
			(cmd[11:8]==4'b1101)	? `IORLW :
			(cmd[11:8]==4'b1100)	? `MOVLW :
			(cmd[11:8]==4'b1000)	? `RETLW :
			(cmd[11:8]==4'b1111)	? `XORLW :
			(cmd==12'b0000_0000_0010) ? `OPTION :
			(cmd==12'b0000_0000_0100) ? `CLRWDT :
			(cmd[11:1]==11'b0000_0000_011)	?`TRIS : `NOP;
endmodule

module mStack(reset,clk,Q,write,read,datin,datout);
	input reset,clk,write,read;
	input Q;
	input [8:0] datin;
	output [8:0] datout;
	reg [8:0] stack1;
	reg [8:0] stack2;
	
	assign datout=stack1;
	
	//Stack1
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			stack1<=9'h0;
		end else begin
			if(Q & write)
				stack1<=datin;
			else if(Q & read)
				stack1<=stack2;
		end
	end
	//Stack2
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			stack2<=9'h0;
		end else begin
			if(Q & write)
				stack2<=stack1;
			else if(Q & read)
				stack2<=9'h0;
		end
	end
endmodule

//Program Counter
module mPCounter(reset,clk,Q,inst,cmd,adr,wr_pc,wr_dat);
	input reset,clk;
	input  Q;			//Start State
	input [4:0] inst;	//Instruction Code
	input [11:0] cmd;	//Command Code(data)
	output [8:0] adr;	//Memory Address
	input wr_pc;		//Write OP to PC
	input [8:0] wr_dat;	//Write Data
	wire call,ret;
	reg [8:0] pc_reg;	//Program Counter
	wire [8:0] sdout;	//Stack Data Output
	
	assign call=(inst==`CALL) ? 1'b1 : 1'b0;
	assign ret=(inst==`RETLW) ? 1'b1 : 1'b0;
	assign adr=pc_reg;
	
	//Program Counter
	always@(posedge clk or posedge reset) begin
		if(reset) begin
			pc_reg<=9'h0;
		end 
		else if(Q) begin
			if(inst==`GOTO) begin
				pc_reg<=cmd[8:0];
			end else if(inst==`CALL) begin
				pc_reg<={1'b0,cmd[7:0]};
			end else if(inst==`RETLW) begin
				pc_reg<=sdout;
			end else if(wr_pc) begin
				pc_reg<=wr_dat;
			end else begin
				pc_reg<=pc_reg+1'b1;
			end
		end
	end
	
	//Stack
	mStack stk(reset,clk,Q,call,ret,pc_reg,sdout);
endmodule

//CommandProcesser
module mCmdProc(cmd,cmd_flash,inst);
	input [11:0] cmd;
	output cmd_flash;
	output [4:0] inst;
	wire [4:0] winst;
	assign inst=winst;
	assign cmd_flash=((winst==`GOTO)||(winst==`CALL)||(winst==`RETLW)) ? 1'b1 : 1'b0;
	mCmdDecord CmdDec(cmd,winst);
endmodule
